#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from opsramp.api import ORapi
import opsramp.binding
from opsramp.rba import Category
import requests_mock


class StaticsTest(unittest.TestCase):
    def setUp(self):
        # Several of the tests need a filename, its contents and
        # the base64 encoded version, so compute all of that up
        # front and store the data for later use. The actual
        # contents are not important so use any file that we know
        # will exist.
        self.testfile = "setup.py"
        with open(self.testfile, "rb") as f:
            self.content_raw = f.read()
        # check it's not empty.
        assert self.content_raw
        self.content_64 = ORapi.b64encode_payload(self.testfile)

    def test_mkAttachment(self):
        tvalues = {"name": self.testfile, "content": self.content_raw}
        actual = Category.mkAttachment(name=tvalues["name"], payload=tvalues["content"])
        expected = {"name": tvalues["name"], "file": tvalues["content"]}
        assert actual == expected

    def test_mkBadParameter(self):
        with self.assertRaises(AssertionError):
            Category.mkParameter(
                name="pname",
                description="pdesc",
                datatype="STRING",
                optional=True
                # deliberately missing default value
            )

    def test_mkGoodParameter(self):
        tvalues = {
            "name": "venue",
            "description": "Where am I today?",
            "type": "STRING",
        }
        actual = Category.mkParameter(
            name=tvalues["name"],
            description=tvalues["description"],
            datatype=tvalues["type"],
        )
        expected = {
            "name": tvalues["name"],
            "description": tvalues["description"],
            "dataType": tvalues["type"],
            "type": "REQUIRED",
            "defaultValue": None,
        }
        assert actual == expected

    def test_mkBadScript(self):
        tvalues = {"name": "sname", "description": "pdesc", "platforms": ["LINUX"]}
        with self.assertRaises(AssertionError):
            Category.mkScript(
                execution_type="Firing squad is not valid",
                name=tvalues["name"],
                description=tvalues["description"],
                platforms=tvalues["platforms"],
                payload_file=self.testfile,
            )
        with self.assertRaises(AssertionError):
            Category.mkScript(
                execution_type="COMMAND",
                name=tvalues["name"],
                description=tvalues["description"],
                platforms=tvalues["platforms"],
                payload="if there is a payload",
                payload_file="cannot have payload_file as well",
            )

    def test_mkGoodCommandScript(self):
        p1 = Category.mkParameter(
            name="venue", description="Where am I today?", datatype="STRING"
        )
        tvalues = {
            "name": "Hello <venue>",
            "description": "Stereotypical rock star intro",
            "platforms": ["LINUX"],
            "type": "COMMAND",
            "payload": 'echo "hello $1"',
            "parameters": [p1],
        }
        actual = Category.mkScript(
            name=tvalues["name"],
            description=tvalues["description"],
            platforms=tvalues["platforms"],
            execution_type=tvalues["type"],
            payload=tvalues["payload"],
            parameters=tvalues["parameters"],
        )
        expected = {
            "name": tvalues["name"],
            "description": tvalues["description"],
            "platforms": tvalues["platforms"],
            "executionType": tvalues["type"],
            "command": tvalues["payload"],
            "parameters": tvalues["parameters"],
        }
        assert actual == expected

    def test_mkGoodPythonScript(self):
        tvalues = {
            "name": "Hello <venue>",
            "description": "Python rock star intro",
            "platforms": ["LINUX"],
            "type": "PYTHON",
            "process_name": "jabberwocky",
            "service_name": "silmarilion",
            "install_timeout": "a really long time",
        }
        actual = Category.mkScript(
            name=tvalues["name"],
            description=tvalues["description"],
            platforms=tvalues["platforms"],
            execution_type=tvalues["type"],
            script_name=self.testfile,
            payload_file=self.testfile,
            install_timeout=tvalues["install_timeout"],
            process_name=tvalues["process_name"],
            service_name=tvalues["service_name"],
        )
        expected = {
            "name": tvalues["name"],
            "description": tvalues["description"],
            "platforms": tvalues["platforms"],
            "executionType": tvalues["type"],
            "attachment": {"name": self.testfile, "file": self.content_64},
            "installTimeout": tvalues["install_timeout"],
            "processName": tvalues["process_name"],
            "serviceName": tvalues["service_name"],
        }
        assert actual == expected


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = "client_for_unit_test"
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.rba = self.client.rba()
        assert "Rba" in str(self.rba)

    def test_categories(self):
        group = self.rba.categories()

        parent = {"id": 123456, "name": "unit-test-parent-category"}
        child = {"id": 789012, "name": "unit-test-child-category"}

        with requests_mock.Mocker() as m:
            # we will be doing two posts so mock both results now.
            url = group.api.compute_url()
            m.post(url, [{"json": parent}, {"json": child}], complete_qs=True)

            # Create a category with no parent ID specified
            actual = group.create(name=parent["name"])
            assert actual == parent

            # Create a category with parent ID specified
            actual = group.create(name=child["name"], parent_uuid=parent["id"])
            assert actual == child

    def test_delete_category(self):
        category_group = self.rba.categories()
        assert category_group
        categoryID = 12345
        expected = ""
        with requests_mock.Mocker() as m:
            url = category_group.api.compute_url(categoryID)
            m.delete(url, text=expected, complete_qs=True)
            actual = category_group.delete(uuid=categoryID)
            assert actual == expected

    def test_update_category(self):
        category_group = self.rba.categories()
        assert category_group
        categoryID = 12345
        updated_name = "some new name"
        expected = {"id": categoryID, "name": updated_name}

        with requests_mock.Mocker() as m:
            url = category_group.api.compute_url()
            m.put(url, json=expected, complete_qs=True)
            actual = category_group.update(categoryID, {"name": updated_name})
            assert actual == expected

    def test_create_script(self):
        this1 = self.rba.categories().category("unit-test-1")
        assert this1
        expected = {"unit": "test"}
        assert expected
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = this1.create(definition=expected)
            assert actual == expected

    def test_update_script(self):
        this1 = self.rba.categories().category("unit-test-1")
        assert this1
        scriptId = 12345
        expected = {"unit": "test"}
        assert expected
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url(scriptId)
            m.post(url, json=expected, complete_qs=True)
            actual = this1.update(uuid=scriptId, definition=expected)
            assert actual == expected

    def test_delete_script(self):
        this1 = self.rba.categories().category("unit-test-1")
        assert this1
        scriptId = 67890
        expected = ""
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url(scriptId)
            m.delete(url, text=expected, complete_qs=True)
            actual = this1.delete(uuid=scriptId)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2020-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import opsramp.binding
import requests_mock


class Metrics(unittest.TestCase):
    def setUp(self):
        fake_url = "mock://api.example.com"
        fake_token = "unit-test-fake-token"
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

    def test_get_metrics(self):
        examples = (
            "tenants/client_1234/metrics/mysql.cluster.status/metricType",
            "search?tenant=client_1234&resource=abcdef&metric=mysql.cluster.status",
        )
        group = self.ormp.metrics()
        for target in examples:
            url = group.api.compute_url(target)
            expected_receive = "fake unit test value " + target
            with requests_mock.Mocker() as m:
                m.get(url, text=expected_receive, complete_qs=True)
                actual = group.get(target)
                assert m.call_count == 1
                assert actual == expected_receive



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
    long_description_content_type = "text/markdown"

with open("requirements.txt", "r") as fh:
    requirements = []
    for line in fh.readlines():
        line = line.strip()
        if line[0] != "#":
            requirements.append(line)

setuptools.setup(
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "ormpcli=opsramp.cli:main",
        ],
    },
)



#!/usr/bin/env python
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from opsramp.base import PathTracker


class TrackerTest(unittest.TestCase):
    def setUp(self):
        self.trkr = PathTracker()
        assert "PathTracker" in str(self.trkr)

    def test_cd(self):
        progression = (
            ("/tmp", "/tmp"),
            ("bossa", "/tmp/bossa"),
            ("nova", "/tmp/bossa/nova"),
            ("george/dragon", "/tmp/bossa/nova/george/dragon"),
            ("/wardrobe/narnia", "/wardrobe/narnia"),
            ("/", ""),
        )
        for pshort, pfull in progression:
            self.trkr.cd(pshort)
            assert self.trkr.fullpath() == pfull

    def test_pushpop(self):
        self.trkr.cd("/themoon")
        home = self.trkr.fullpath()

        city = "/europe/germany/berlin"
        street = "Alexanderplatz"
        both = city + "/" + street

        a_pushd = 1
        a_popd = 2
        progression = (
            (a_pushd, city, city),
            (a_pushd, street, both),
            (a_popd, None, city),
            (a_pushd, street, both),
            (a_pushd, "/logs", "/logs"),
            (a_popd, None, both),
            (a_popd, None, city),
            (a_popd, None, home),
        )
        assert self.trkr.fullpath() == home
        for action, pshort, pfull in progression:
            if action == a_pushd:
                self.trkr.pushd(pshort)
            elif action == a_popd:
                assert not pshort
                self.trkr.popd()
            else:
                raise AssertionError("unexpected action %s" % action)
            assert self.trkr.fullpath() == pfull

    def test_reset(self):
        self.trkr.cd("/solarsystems/milkyway")
        self.trkr.reset()
        assert self.trkr.fullpath() == ""

    def test_clone(self):
        place1 = "/saturn/atmosphere"
        self.trkr.cd(place1)
        # check that cd in a clone doesn't affect the original.
        other = self.trkr.clone()
        place2 = "/jupiter/satellites/moons/ganymede"
        other.cd(place2)
        assert self.trkr.fullpath() == place1
        assert other.fullpath() == place2

    def test_fullpath(self):
        # check that it doesn't "cd" to the specified suffix, just appends it.
        self.trkr.cd("my/hovercraft")
        original = self.trkr.fullpath()
        suffix = "is/full/of/eels"
        other = self.trkr.fullpath(suffix)
        assert other == original + "/" + suffix
        assert self.trkr.fullpath() == original
        # test with a / at the start, which should not append (nor cd there).
        suffix = "/full/path/not/relative"
        other = self.trkr.fullpath(suffix)
        assert other == suffix
        assert self.trkr.fullpath() == original



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# msp.py
# Classes related to partner-level actions.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

from opsramp.api import ORapi


class Clients(ORapi):
    def __init__(self, parent):
        super(Clients, self).__init__(parent.api, "clients")

    def get(self, suffix="/minimal"):
        return self.api.get(suffix)

    def search(self, query_string=""):
        # For historical reasons the caller is allowed to omit the
        # queryString prefix, so add it if necessary.
        key = "queryString="
        if query_string and key not in query_string:
            query_string = key + query_string
        return super(Clients, self).search(pattern=query_string)

    def create(self, definition):
        assert "name" in definition
        assert "address" in definition
        assert "timeZone" in definition
        assert "country" in definition
        return self.api.post("", json=definition)

    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    def activate(self, uuid):
        return self.api.post("%s/activate" % uuid)

    def suspend(self, uuid):
        return self.api.post("%s/suspend" % uuid)

    def terminate(self, uuid):
        return self.api.post("%s/terminate" % uuid)

    # Helper functions to create the complex structures that OpsRamp
    # uses to manipulate client definitions.
    @staticmethod
    def mkHours(
        day_start=None,
        day_end=None,
        week_start=2,
        week_end=6,
        sms_voice_notification=False,
    ):
        if not day_start:
            day_start = datetime.time(9, 0)
        if not day_end:
            day_end = datetime.time(17, 0)
        retval = {
            "businessStartHour": day_start.hour,
            "businessStartMin": day_start.minute,
            "businessEndHour": day_end.hour,
            "businessEndMin": day_end.minute,
            "businessDayStart": int(week_start),
            "businessDayEnd": int(week_end),
            "smsVoiceNotification": bool(sms_voice_notification),
        }
        return retval

    # A helper function to create the complex structures that OpsRamp
    # uses to define a new client. There are lots of optional fields and
    # potential gotchas here and we guard against *some* of them.
    @staticmethod
    def mkClient(name, address, time_zone, country, hours=None):
        retval = {
            "name": name,
            "address": address,
            "timeZone": time_zone,
            "country": country,
        }
        if hours:
            retval["clientDetails"] = hours
        # TODO there are lots and lots more optional fields that we
        # will probably need to cater for in the fullness of time.
        return retval



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# monitoring.py
# Classes related to monitoring templates and similar things.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from opsramp.api import ORapi


class Monitoring(ORapi):
    def __init__(self, parent):
        super(Monitoring, self).__init__(parent.api, "monitoring")

    def templates(self):
        return Templates(self)


class Templates(ORapi):
    def __init__(self, parent):
        super(Templates, self).__init__(parent.api, "templates")



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import opsramp.binding
import opsramp.msp
import opsramp.rba
import yaml


CATEGORY_NAME = "python-opsramp test category"


def main():
    OPSRAMP_URL = os.environ["OPSRAMP_URL"]
    TENANT_ID = os.environ["OPSRAMP_TENANT_ID"]
    KEY = os.environ["OPSRAMP_KEY"]
    SECRET = os.environ["OPSRAMP_SECRET"]

    ormp = opsramp.binding.connect(OPSRAMP_URL, KEY, SECRET)

    # Print some random global "stuff" just to show that we can.
    cfg = ormp.config()
    print("alert types", cfg.get_alert_types())
    print("channels", cfg.get_channels())
    print("countries", len(cfg.get_countries()))
    print("timezones", len(cfg.get_timezones()))
    print("alert technologies", len(cfg.get_alert_technologies()))
    print("nocs", len(cfg.get_nocs()))
    print("device types", len(cfg.get_device_types()))

    # Focus on a specific tenant.
    tenant = ormp.tenant(TENANT_ID)

    if tenant.is_client():
        print("List the First Response Policies on tenant", TENANT_ID)
        group = tenant.first_response()
        found = group.search()
        print(found["totalResults"], "first response policies")
        print(yaml.dump(found["results"], default_flow_style=False))

    if tenant.is_client():
        print("List the Service Maps on tenant", TENANT_ID)
        group = tenant.service_maps()
        found = group.get(minimal=True)
        print(len(found), "service maps")
        print(yaml.dump(found, default_flow_style=False))

    print("List the RBAC permission sets on tenant", TENANT_ID)
    group = tenant.permission_sets()
    found = group.search()
    print(found["totalResults"], "permission_sets")
    print(yaml.dump(found["results"], default_flow_style=False))

    print("List the RBAC roles on tenant", TENANT_ID)
    group = tenant.roles()
    found = group.search()
    print(found["totalResults"], "roles")
    print(yaml.dump(found["results"], default_flow_style=False))

    print("List the Sites on tenant", TENANT_ID)
    group = tenant.sites()
    found = group.search()
    print(found["totalResults"], "sites")
    print(yaml.dump(found["results"], default_flow_style=False))

    print("List the integrations on tenant", TENANT_ID)
    integs = tenant.integrations()
    group = integs.itypes()
    found = group.search()
    print(found["totalResults"], "integration types")
    print(yaml.dump(found["results"]))

    print("Define new custom integrations on", TENANT_ID)
    group = integs.instances()
    for atype in ("OAUTH2", "BASIC"):
        newcint = group.mkCustom(
            display_name="Example %s integration" % atype, inbound_auth_type=atype
        )
        print(yaml.dump(newcint))
        # uncomment the following lines to actually create the integration.
        # if tenant.is_client():
        #     resp = group.create('CUSTOM', newcint)
        #     group.redact_response(resp)
        #     print(resp)

    found = group.search()
    print(found["totalResults"], "integration instances")
    # "found" contains a complete description of each integration but let's
    # pull them individually anyway and assert that this gives same result.
    print("GET the integrations individually and compare")
    for i in found["results"]:
        print(
            "...",
            i["id"],
            i["integration"]["id"],
            '"' + i.get("displayName", "<no name>") + '"',
        )
        direct = group.get(i["id"])
        assert direct == i

    # Retrieve the agent installation script for this client. The string
    # that OpsRamp returns will contain keys for the client so just print
    # its length and first line to show that we got something.
    if tenant.is_client():
        print("agent installation script for client", TENANT_ID)
        agent_script = tenant.get_agent_script()
        print("length", len(agent_script))
        print(agent_script.split("\n")[0])
        print("")
    else:
        print("List the clients of tenant", TENANT_ID)
        group = tenant.clients()
        clist = group.get()
        print(len(clist), "clients")
        print(yaml.dump(clist, width=150))
        for c in group.get():
            print("client", yaml.dump(c, width=150).strip())
            resp = group.get(c["uniqueId"])
            print("createdBy", yaml.dump(resp["createdBy"], width=150))
        print("Exercise the create-client code")
        cdef = group.mkClient(
            name="test client 99",
            address="Death Valley",
            time_zone="America/Los_Angeles",
            country="United States",
        )
        print(yaml.dump(cdef))
        # uncomment these lines to actually create the client.
        # resp = group.create(cdef)
        # print(resp)

    print("List the Resources on tenant", TENANT_ID)
    if tenant.is_client():
        group = tenant.resources()
        found = group.minimal()
        print(found["totalResults"], "resources")
        print(yaml.dump(found["results"]))
    else:
        print("<clients only>")

    print("List the monitoring templates on tenant", TENANT_ID)
    monitoring = tenant.monitoring()
    templates = monitoring.templates()
    resp = templates.search()
    print(resp["totalResults"], "templates")
    # There are typically hundreds of templates so don't YAML dump them.
    for t in resp["results"]:
        print("...", t["name"])

    # list the RBA script categories and find or create the one we want.
    print("List the RBA categories on tenant", TENANT_ID)
    rba = tenant.rba()
    group = rba.categories()
    clist = group.get()
    print(len(clist), "categories")
    print(yaml.dump(clist))
    for c in clist:
        if c["name"] == CATEGORY_NAME:
            cid = c["id"]
            break
    else:
        # resp = group.create(CATEGORY_NAME)
        # cid = resp['id']
        print("Category {} not found".format(CATEGORY_NAME))
        cid = None

    if cid:
        print("Scripts in category", cid)
        cobj = group.category(cid)
        slist = cobj.get()
        print(len(slist), "scripts")
        for s in slist:
            print("...", s["id"], s["name"])
            direct = cobj.get(s["id"])
            assert s == direct

        # Create a new RBA script in this category with one parameter. You can
        # see from the "payload" that this gets passed to the script as $1
        p1 = cobj.mkParameter(
            name="venue", description="Where am I today?", datatype="STRING"
        )
        print("Parameter definition struct")
        print(yaml.dump(p1))
        s1 = cobj.mkScript(
            name="Hello <venue>",
            description="Stereotypical rock star intro",
            platforms=["LINUX"],
            execution_type="COMMAND",
            payload='echo "hello $1"',
            parameters=[p1],
        )
        print("Script definition struct")
        print(yaml.dump(s1))
        # uncomment these lines to actually create the script.
        # resp = cobj.create(s1)
        # print(resp)

    print("Management policies on tenant", TENANT_ID)
    group = tenant.policies()
    plist = group.get()
    print(len(plist), "policies")
    for p in plist:
        rules = p["rules"]
        actions = p["actions"]
        print(
            "...",
            p["id"],
            p["uid"],
            "name",
            p["name"],
            "rules",
            len(rules),
            "actions",
            len(actions),
        )
        for r in rules:
            print("...... rule", r)
        for a in actions:
            print("...... action", a["action"])
        # plist contains a complete description of each policy
        # but let's pull them individually anyway and assert that
        # this gives the same result.
        direct = group.get(p["id"])
        assert p == direct
    print("")

    print("Credential sets on tenant", TENANT_ID)
    if tenant.is_client():
        cs = tenant.credential_sets()
        resp = cs.get()
        print(resp["totalResults"], "credential sets")
        print(yaml.dump(resp["results"], default_flow_style=False))
    else:
        print("<clients only>")

    print("Discovery Profiles on tenant", TENANT_ID)
    if tenant.is_client():
        group = tenant.discovery()
        profiles = group.get()
        print(len(profiles), "profiles")
        for profile in profiles:
            uuid = profile["id"]
            name = profile["name"]
            print("...", uuid, '"%s"' % name)
            print(yaml.dump(profile))
            direct = group.get(uuid)
            assert direct == profile
        print("")
    else:
        print("<clients only>")

    print("Escalation Policies on tenant", TENANT_ID)
    group = tenant.escalations()
    found = group.search()
    print(found["totalResults"], "escalation policies")
    print(yaml.dump(found["results"], default_flow_style=False))

    print("Management profiles on tenant", TENANT_ID)
    group = tenant.mgmt_profiles()
    found = group.search()
    print(found["totalResults"], "management profiles")
    print(yaml.dump(found["results"], default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A Python language binding for the OpsRamp REST API.
#
# integrations.py
# Classes related to Integrations.
#
# (c) Copyright 2019-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from opsramp.api import ORapi

"""
POST install/{intgld} e.g. CUSTOM

GET installed/search?queryString=
GET installed/{installedIntgld}
GET installed/{installedIntgld}/configFile/Kubernetes
DELETE installed/{installedIntgld}
POST installed/{installedIntgld}
POST installed/{installedIntgld}/notifier
POST installed/{installedIntgld}/enable
POST installed/{installedIntgld}/disable
POST installed/{installedIntgld}/inbound/authentication
POST installed/{installedIntgld}/mappingAttr

GET available/search?queryString=
GET available/{intgld}/emailProps/{entityType}
GET available/{intgld}/mappingAttr/{entityType}
"""


class Integrations(ORapi):
    def __init__(self, parent):
        super(Integrations, self).__init__(parent.api, "integrations")

    def itypes(self):
        return Types(self)

    def instances(self):
        return Instances(self)

    def available(self):
        # Compatibility function because this is the name that the
        # OpsRamp API docs use for this data set.
        return self.itypes()

    def installed(self):
        # Compatibility function because this is the name that the
        # OpsRamp API docs use for this data set.
        return self.instances()


class Types(ORapi):
    def __init__(self, parent):
        super(Types, self).__init__(parent.api, "available")


class Instances(ORapi):
    def __init__(self, parent):
        super(Instances, self).__init__(parent.api, "installed")
        # The OpsRamp "create instance" API endpoint is in an
        # atypical place in the tree so need to compute it.
        self.creator_api = parent.api.clone()
        self.creator_api.chroot("install")

    def get_kubernetes_configuration(self, uuid):
        return self.api.get("%s/configFile/Kubernetes" % uuid)

    def create(self, type_name, definition):
        resp = self.creator_api.post(type_name, json=definition)
        return resp

    def delete(self, uuid, uninstall_reason="<Not specified>"):
        json_payload = {"uninstallReason": uninstall_reason}
        return self.api.delete(suffix=uuid, json=json_payload)

    def update(self, uuid, definition):
        return self.api.post("%s" % uuid, json=definition)

    def set_auth_type(self, uuid, auth_type):
        assert auth_type in ("OAUTH2", "WEBHOOK", "BASIC")
        settings = {"authType": auth_type}
        return self.api.post("%s/inbound/authentication" % uuid, json=settings)

    def enable(self, uuid):
        return self.api.post("%s/enable" % uuid)

    def disable(self, uuid):
        return self.api.post("%s/disable" % uuid)

    def notifier(self, uuid, definition):
        # Create or update installed integration base notifier
        return self.api.post("%s/notifier" % uuid, json=definition)

    # A helper function that extracts the authentication types from typical
    # integration instance response structs.
    @staticmethod
    def auth_type(resp):
        in_auth = (
            resp.get("inboundConfig", {})
            .get("authentication", {})
            .get("authType", None)
        )
        out_auth = (
            resp.get("outboundConfig", {})
            .get("authentication", {})
            .get("authType", None)
        )
        return (in_auth, out_auth)

    # A helper function that looks for login credentials in typical
    # integration instance response structs and redacts them.
    @staticmethod
    def redact_response(resp):
        for key in ("inboundConfig", "outboundConfig"):
            config = resp.get(key, {})
            auth = config.get("authentication", {})
            if "token" in auth:
                auth["token"] = "REDACTED"
            for creds in auth.get("apiKeyPairs", {}):
                for i in creds.keys():
                    creds[i] = "REDACTED"
        return resp

    # Helper functions to create the complex structures that OpsRamp
    # uses to define new integration instances. There are lots of
    # optional fields and potential gotchas here and we guard against
    # *some* of them.
    @staticmethod
    def mkBase(display_name, logo_fname=None):
        assert display_name
        retval = {
            "displayName": display_name,
        }
        if logo_fname:
            payload = ORapi.b64encode_payload(logo_fname)
            retval["logo"] = {"name": os.path.basename(logo_fname), "file": payload}
        return retval

    @staticmethod
    def mkEmailAlert(display_name, logo_fname=None):
        assert display_name
        retval = Instances.mkBase(display_name, logo_fname)
        return retval

    @staticmethod
    def mkCustom(
        display_name, logo_fname=None, parent_uuid=None, inbound_auth_type=None
    ):
        assert display_name
        retval = Instances.mkBase(display_name, logo_fname)
        if parent_uuid:
            retval["parentIntg"] = {"id": parent_uuid}
        if inbound_auth_type:
            assert inbound_auth_type in ("OAUTH2", "BASIC")
            retval["inboundConfig"] = {
                "authentication": {"authType": inbound_auth_type}
            }
        return retval

    @staticmethod
    def mkAzureARM(
        display_name, arm_subscription_id, arm_tenant_id, arm_client_id, arm_secret_key
    ):
        assert display_name
        retval = {
            "displayName": display_name,
            "credential": {
                "credentialType": "AZURE",
                "AzureType": "ARM",
                "SubscriptionId": arm_subscription_id,
                "TenantId": arm_tenant_id,
                "ClientID": arm_client_id,
                "SecretKey": arm_secret_key,
            },
        }
        return retval

    @staticmethod
    def mkAzureASM(display_name, arm_subscription_id, arm_mgmt_cert, arm_keystore_pass):
        assert display_name
        retval = {
            "displayName": display_name,
            "credential": {
                "credentialType": "AZURE",
                "AzureType": "ASM",
                "SubscriptionId": arm_subscription_id,
                "ManagementCertificate": arm_mgmt_cert,
                "KeystorePassword": arm_keystore_pass,
            },
        }
        return retval

    @staticmethod
    def mkBaseNotifier(
        api_type,
        base_uri,
        auth_type="OAUTH2",
        grant_type="CLIENT_CREDENTIALS",
        access_token_uri=None,
        api_key=None,
        api_secret=None,
        user_name=None,
        password=None,
    ):
        assert api_type in ("REST_API", "SOAP_API")
        assert base_uri
        assert auth_type in ("NONE", "BASIC", "OAUTH2")
        retval = {"type": api_type, "baseURI": base_uri, "authType": auth_type}
        if auth_type != "NONE":
            retval["grantType"] = grant_type
            if auth_type == "OAUTH2":
                assert grant_type in ("CLIENT_CREDENTIALS", "PASSWORD")
                # empty string is ok but omitting these isn't.
                assert access_token_uri is not None
                assert api_key is not None
                assert api_secret is not None
                retval["accessTokenURI"] = access_token_uri
                retval["apiKey"] = api_key
                retval["apiSecret"] = api_secret
            if grant_type == "PASSWORD":
                # empty string is ok but omitting these isn't.
                assert user_name is not None
                assert password is not None
                retval["userName"] = user_name
                retval["password"] = password
        return retval

    # mkValueMap(('attrValue', 'tenantAttrValue'),
    #            (('client_113', '22cdbc5bb401d737b088c9'),
    #             ('client_843', '66cdbc5bb401d737b088c9'))
    @staticmethod
    def mkValueMap(labels, value_pairs):
        ormpname, othername = labels
        retval = [{ormpname: x[0], othername: x[1]} for x in value_pairs]
        return retval
